"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validatePayload = ({ schemaValidator, payload, propertiesToValidate, }) => __awaiter(void 0, void 0, void 0, function* () {
    const errors = {};
    for (const propertyToValidate of propertiesToValidate) {
        const propertySchema = schemaValidator.schema[propertyToValidate];
        try {
            yield (propertySchema === null || propertySchema === void 0 ? void 0 : propertySchema.yupSchema.validate(payload[propertyToValidate], propertySchema.validateOptions));
        }
        catch (yupValidationError) {
            errors[propertyToValidate] = buildErrorPayload({
                yupValidationError,
                errorMessages: schemaValidator.errorMessages,
            });
        }
    }
    return Object.keys(errors).length ? errors : null;
});
const buildErrorPayload = ({ errorMessages, yupValidationError, }) => {
    const yupValidationResultErrors = yupValidationError.inner && yupValidationError.inner.length > 0 ? yupValidationError.inner : [yupValidationError];
    return yupValidationResultErrors.reduce((errorsResult, { path: propertyPath, message: yupErrorMessage }) => {
        if (errorsResult.find(({ propertyPath: accPropertyPath }) => accPropertyPath === propertyPath)) {
            return errorsResult;
        }
        const errorMessage = errorMessages ? errorMessages[yupErrorMessage] : { message: yupErrorMessage };
        return errorsResult.concat(Object.assign(Object.assign({}, errorMessage), { propertyPath }));
    }, []);
};
//# sourceMappingURL=schema-validator.js.map